﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using CsvHelper;

namespace HighlightDispDemo
{
    public partial class MainWindow : Window
    {
        private IEnumerable<LinuxInfo> _listData;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            TextReader tr = new StreamReader("linux.csv", Encoding.UTF8);
            CsvReader csv = new CsvReader(tr);
            csv.Configuration.RegisterClassMap<LinuxMap>();
            csv.Configuration.HasHeaderRecord = false;
            _listData = csv.GetRecords<LinuxInfo>().ToList();
            lvContent.ItemsSource = _listData;
        }

        private void btnSearch_Click(object sender, RoutedEventArgs e)
        {
            HighlightContent.Mode = radioFullMatch.IsChecked == true
                                        ? HighlightContentMode.FullMatch
                                        : HighlightContentMode.AnyKey;
            HighlightContent.ToHighlight = tbKey.Text;
            IEnumerable<LinuxInfo> todisp;
            if (HighlightContent.Mode==HighlightContentMode.FullMatch)
            {
                if (!string.IsNullOrEmpty(HighlightContent.ToHighlight))
                {
                    todisp = _listData.Where(r => (r.Desktop + "\0" + r.FileSystem + "\0" + r.OsName).IndexOf(HighlightContent.ToHighlight, StringComparison.OrdinalIgnoreCase) != -1);
                }
                else
                {
                    todisp = _listData;
                }
            }
            else
            {
                string[] keys = HighlightContent.ToHighlight.Split(new[] {' '}, StringSplitOptions.RemoveEmptyEntries);
                if (keys.Length > 0)
                {
                    todisp = _listData.Where(r => keys.Any(key => (r.Desktop + "\0" + r.FileSystem + "\0" + r.OsName).IndexOf(key, StringComparison.OrdinalIgnoreCase) != -1));
                }
                else
                {
                    todisp = _listData;
                }
            }
            lvContent.ItemsSource = todisp;
        }
    }
}
